@@ -27,6 +27,11 @@ class DBCreator(object):
         self.features = gfffile
         self.verbose = verbose
 
+    def msg(self, s):
+        if self.verbose:
+            sys.stderr.write(s)
+            sys.stderr.flush()
+
     def populate_from_features(self, features):
         pass
 
@@ -93,15 +98,12 @@ class GFFDBCreator(DBCreator):
         nfeatures = float(self.nfeatures)
         last_perc = 0
         for i, feature in enumerate(features):
-            if self.verbose:
-                perc = int(i / nfeatures * 100)
-                if perc != last_perc:
-                    msg = 'Populating features table and first-order '
-                    msg += 'relations: %d (%d%%)' % (i, perc)
-
-                    sys.stderr.write('\r' + msg)
-                    sys.stderr.flush()
-                last_perc = perc
+            perc = int(i / nfeatures * 100)
+            if perc != last_perc:
+                msg = 'Populating features table and first-order '
+                msg += 'relations: %d (%d%%)' % (i, perc)
+                self.msg('\r' + msg)
+            last_perc = perc
 
             if not feature.id:
                 new_id = '%s:%s:%s-%s:%s' % (feature.featuretype,
@@ -139,14 +141,11 @@ class GFFDBCreator(DBCreator):
         self.conn.commit()
 
         t1 = time.time()
-        if self.verbose:
-            sys.stderr.write('...done in %.1fs\n' % (t1 - t0))
-            sys.stderr.flush()
+        self.msg('...done in %.1fs\n' % (t1 - t0))
 
     def update_relations(self):
         t0 = time.time()
-        if self.verbose:
-            sys.stderr.write('Updating 2nd-order relations...')
+        self.msg('Updating 2nd-order relations...')
         c = self.conn.cursor()
 
         # create 2 more cursors so we can iterate over one while querying on
@@ -170,24 +169,13 @@ class GFFDBCreator(DBCreator):
         fout = open(tmp, 'w')
         for parent in c:
             parent = parent[0]
-
-            # Here we get the first-level child from the initial import.   This
-            # data was contained in the "Parent=" attribute of each GFF
-            # feature.
-            c2.execute('''
-            SELECT child FROM relations
-            WHERE parent = ? AND level=1''', (parent,))
-
-            # For each of those children, we get *their* children -- so the
-            # 'grandchildren' of the original parent.
-            for child in c2:
-                child = child[0]
-                c3.execute('''
-                SELECT child FROM relations
-                WHERE parent = ? AND level=1''', (child,))
-                for grandchild in c3:
-                    grandchild = grandchild[0]
-                    fout.write('%s\t%s\n' % (parent, grandchild))
+            c2.execute("""
+            SELECT child FROM relations WHERE parent IN
+                (SELECT child FROM relations WHERE parent=?)
+            """, (parent,))
+            for grandchild in c2:
+                grandchild = grandchild[0]
+                fout.write('%s\t%s\n' % (parent, grandchild))
         fout.close()
 
         # Import the "grandchild" file into the relations table
@@ -198,15 +186,11 @@ class GFFDBCreator(DBCreator):
             VALUES (?,?,?)''', (parent, child, 2))
 
         t1 = time.time()
-        if self.verbose:
-            sys.stderr.write('done in %.1fs\n' % (t1 - t0))
-            sys.stderr.flush()
+        self.msg('done in %.1fs\n' % (t1 - t0))
 
         # Re-create indexes.
         # TODO: performance testing to see how much of a difference these make
-        if self.verbose:
-            sys.stderr.write('Creating indexes...')
-            sys.stderr.flush()
+        self.msg('Creating indexes...')
         c.execute('drop index childindex')
         c.execute('drop index parentindex')
         c.execute('create index parentindex on relations (parent)')
@@ -220,9 +204,7 @@ class GFFDBCreator(DBCreator):
         self.conn.commit()
         os.unlink(tmp)
         t2 = time.time()
-        if self.verbose:
-            sys.stderr.write('done in %.1fs\n' % (t2 - t1))
-            sys.stderr.flush()
+        self.msg('done in %.1fs\n' % (t2 - t1))
 
 
 class GTFDBCreator(DBCreator):
@@ -232,9 +214,20 @@ class GTFDBCreator(DBCreator):
 
     def populate_from_features(self, features):
         t0 = time.time()
+        self.msg("Populating features and relations tables...")
         self.drop_indexes()
+        nfeatures = float(self.nfeatures)
+        last_perc = 0
         c = self.conn.cursor()
-        for feature in features:
+        for i, feature in enumerate(features):
+
+            perc = int(i / nfeatures * 100)
+            if perc != last_perc:
+                msg = 'Populating features and relations '
+                msg += 'table: %d (%d%%) ' % (i, perc)
+                self.msg('\r' + msg)
+            last_perc = perc
+
             parent = feature.attributes['transcript_id']
             grandparent = feature.attributes['gene_id']
 
@@ -281,19 +274,40 @@ class GTFDBCreator(DBCreator):
                        feature.frame,
                        feature._str_attributes))
         self.conn.commit()
+        t1 = time.time()
+        self.msg('done in %.1fs\n' % (t1 - t0))
 
     def update_relations(self):
         self.drop_indexes()
         c = self.conn.cursor()
         c2 = self.conn.cursor()
+        t0 = time.time()
+        self.msg("Creating temporary indexes...")
         c.execute('CREATE INDEX ids ON features (id)')
         c.execute('CREATE INDEX parentindex ON relations (parent)')
         c.execute('CREATE INDEX childindex ON relations (child)')
+        t1 = time.time()
+        self.msg('done in %.1fs\n' % (t1 - t0))
 
+        t0 = time.time()
+        self.msg(
+                "Inferring gene and transcript extents; saving unique "
+                "to tempfile... ")
         tmp = tempfile.mktemp()
         fout = open(tmp, 'w')
         c.execute("SELECT DISTINCT parent FROM relations")
-        for parent in c:
+        parents = list(c)
+        nparents = float(len(parents))
+        last_perc = 0
+        for i, parent in enumerate(parents):
+
+            perc = int(i / nparents * 100)
+            if perc != last_perc:
+                msg = 'Inferring gene and transcript extents: '
+                msg += '%d (%d%%)...' % (i, perc)
+                self.msg('\r' + msg)
+            last_perc = perc
+
             parent = parent[0]
             c2.execute("""
                        SELECT min(start), max(stop), level, strand, chrom
@@ -329,7 +343,7 @@ class GTFDBCreator(DBCreator):
             # parents at level 1 are mRNAs.
             #
             # WARNING: this does NOT account for non-coding RNAs -- they will
-            # still be called "mRNA"
+            # still be called "mRNA".
 
             if level == 1:
                 featuretype = 'mRNA'
@@ -347,6 +361,12 @@ class GTFDBCreator(DBCreator):
 
         # Now that the file has been created and we're done querying for the
         # parents, slurp in the file and insert everything into the db.
+
+        t1 = time.time()
+        self.msg('done in %.1fs\n' % (t1 - t0))
+
+        t0 = time.time()
+        self.msg('Importing inferred features...')
         fin = open(fout.name)
         for line in fin:
             score = '.'
@@ -359,11 +379,17 @@ class GTFDBCreator(DBCreator):
                       """, line.strip().split('\t') + [score, source, frame])
 
         self.conn.commit()
+        t1 = time.time()
+        self.msg('done in %.1fs\n' % (t1 - t0))
         os.remove(tmp)
 
+        t0 = time.time()
+        self.msg('Creating index...')
         c.execute('DROP INDEX ids')
         c.execute('CREATE INDEX ids ON features (id)')
         self.conn.commit()
+        t1 = time.time()
+        self.msg('done in %.1fs\n' % (t1 - t0))
 
 
 class FeatureDB:
@@ -419,8 +445,16 @@ class FeatureDB:
 
     def attribute_search(self, text, featuretype='gene'):
         """
-        Looks for *text* within the "attributes" field of features of
-        *featuretype* ('gene' by default).  Useful for looking up genes based
+        Search for a string in features' attributes.
+
+        :param text: String to search for, case-insensitive.
+        :param featuretype: Restrict search to a particular featuretype;
+                            default is `"gene"`
+
+        :rtype: Iterator of :class:`gffutils.Feature` objects
+
+        Looks for `text` within the attributes field of features of
+        `featuretype` ('"gene"' by default).  Useful for looking up genes based
         on symbol or name rather than accession number.  Returns a list of
         matching Features.
 
@@ -432,17 +466,23 @@ class FeatureDB:
                   %s from features where attributes
                   like "%s" and featuretype = ?
                   ''' % (self.SELECT, text,), (featuretype,))
-        results = []
         for result in c.fetchall():
-            results.append(self._newfeature(*result))
-        return results
+            yield self._newfeature(*result)
 
     def features_of_type(self, featuretype, chrom=None, start=None, stop=None,
             strand=None):
         """
-        Returns an iterator of GFFFeature objects that have the feature type
-        *featuretype*.  You can optionally specify chrom, strand, start, and
-        stop filters.
+        Retrieve feature of a specific feature type (gene, exon, etc.)
+
+        :param featuretype: feature type to return
+        :param chrom: Optional chromsome to restrict results
+        :param start: Optional start position to restrict results
+        :param stop: Optional stop position to restric results
+        :rtype: Iterator of :class:`gffdb.Feature` objects
+
+        Returns an iterator of :class:`gffutils.Feature` objects that have the
+        feature type `featuretype`.  You can optionally specify chrom, strand,
+        start, and stop filters.
 
         For example, to get all exons within a range::
 
@@ -457,9 +497,9 @@ class FeatureDB:
             # genes on + strand of chrX
             self.features_of_type('gene',chrom='chrX', strand='+')
 
-        For more complicated stuff, (e.g., filtering by multiple chromosomes)
-        you'll probably want to use self.execute() directly and write your own
-        SQL queries . . . or just call this method iteratively.
+        For more complicated operations, (e.g., filtering by multiple
+        chromosomes) you'll probably want to use self.execute() directly and
+        write your own SQL queries . . . or just call this method iteratively.
         """
         filter_clause = ''
         if chrom is not None:
@@ -486,8 +526,9 @@ class FeatureDB:
 
     def featuretypes(self):
         """
-        Returns an iterator of the different feature types found in the
-        database.
+        Iterator of featuretypes found in the database.
+
+        :rtype: Iterator of :class:`Feature` objects
         """
         c = self.conn.cursor()
         c.execute('''
@@ -509,15 +550,18 @@ class FeatureDB:
         for i in c:
             yield self._newfeature(*i)
 
-    def exonic_bp(self, id, ignore_strand=False):
+    def exonic_bp(self, feature, ignore_strand=False):
         """
-        Merges all exons of the gene *id* and sums the total bp.
-        *ignore_strand* is passed to merge_features(). Useful for calculating
+        Convenience function for calculating unique exonic bp for a gene across
+        all isoforms.
+
+        Merges all exons of the gene `id` and sums the total bp.
+        `ignore_strand` is passed to merge_features(). Useful for calculating
         RPKM for full gene models.
         """
-        if isinstance(id, Feature):
-            id = id.id
-        exons = self.children(id, featuretype='exon', level=2)
+        if isinstance(feature, Feature):
+            feature = feature.id
+        exons = self.children(feature, featuretype='exon', level=2)
         exons = list(exons)
         if len(exons) == 0:
             return 0
@@ -978,7 +1022,8 @@ class FeatureDB:
           nonzero then there must be multiple features for the same codon
         * frame:
             * 0 = starts with whole codon at 5'-most base
-            * 1 = one extra base (the last of a codon) before the first whole codon
+            * 1 = one extra base (the last of a codon) before
+                  the first whole codon
             * 2 = two extra bases
 
         """
@@ -1074,8 +1119,9 @@ class FeatureDB:
                                      frame='0',
                                      attributes=attributes)
 
-                    # write out the first CDS, but only if a multi-CDS gene -- since
-                    # single-CDS genes need to have stop_codon coords trimmed
+                    # write out the first CDS, but only if a multi-CDS gene --
+                    # since single-CDS genes need to have stop_codon coords
+                    # trimmed
                     nextframe = 0
                     if len(cdss) > 1:
                         yield Feature(chrom=transcript.chrom,
@@ -1137,7 +1183,8 @@ class FeatureDB:
                                      frame='0',
                                      attributes=attributes)
 
-                    # first CDS is last sorted; only deal with it here a multi-CDS gene
+                    # first CDS is last sorted; only deal with it here a
+                    # multi-CDS gene
                     nextframe = 0
                     if len(cdss) > 1:
                         yield Feature(chrom=transcript.chrom,
